﻿using Microsoft.Extensions.Logging;
using System.Windows.Forms;

namespace AirBomber
{
    public partial class FormMapWithSetAirplanes : Form
    {
        /// <summary>
        /// Словарь для выпадающего списка
        /// </summary>
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Простая карта", new SimpleMap() },
            { "Карта со стенами", new WallMap() },
        };
        /// <summary>
        /// Объект от коллекции карт
        /// </summary>
        private readonly MapsCollection _mapsCollection;
        private GeneratorAirplane<EntityAirplane, IAirplaneEngines> _generatorAirplane;
        /// <summary>
        /// Все объекты удаленные с каких либо карт
        /// </summary>
        private LinkedList<IDrawningObject> _deletedObjects;
        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMapWithSetAirplanes(ILogger<FormMapWithSetAirplanes> logger)
        {
            InitializeComponent();
            _generatorAirplane = new(100, 100);
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            _logger = logger;
            _deletedObjects = new();
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }

        private string NameMap => listBoxMaps.SelectedItem?.ToString() ?? string.Empty;
        /// <summary>
        /// Заполнение listBoxMaps
        /// </summary>
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;

            listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation("При добавлении карты {0}", comboBoxSelectorMap.SelectedIndex == -1 ? "Не была выбрана карта" : "Не была названа карта");
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Нет карты с названием: {0}", textBoxNewMapName.Text);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation("Добавлена карта {0}", textBoxNewMapName.Text);
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[NameMap].ShowSet();
            _logger.LogInformation("Был осуществлен переход на карту под названием: {0}", listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }

            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(NameMap);
                _logger.LogInformation("Удалена карта {0}", listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
        }
        /// <summary>Добавление самолета на карту</summary>
        /// <param name="airplane">Самолет</param>
        /// <param name="text">Текст MessageBox если самолета нет</param>
        /// <param name="caption">Заголовок MessageBox если самолета нет</param>
        private void AddAirplaneInMap(DrawningObject? airplane, string? text = null, string? caption = null)
        {
            try
            {
                if (listBoxMaps.SelectedIndex == -1)
                {
                    MessageBox.Show("Перед добавлением объекта необходимо создать карту");
                }
                else if (airplane == null)
                {
                    MessageBox.Show(text, caption);
                }
                else if ((_mapsCollection[NameMap] + airplane) == -1)
                {
                    MessageBox.Show("Не удалось добавить объект");
                    _logger.LogInformation("Не удалось добавить объект");
                }
                else
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapsCollection[NameMap].ShowSet();
                    _logger.LogInformation("Добавлен объект {@Airplane}", (DrawningAirplane)airplane);
                }
            }
            catch (StorageOverflowException ex)
            {
                _logger.LogWarning("Ошибка переполнения хранилища: {0}", ex.Message);
                MessageBox.Show($"Ошибка переполнения хранилища: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[NameMap].MoveObject(dir);
        }
        /// <summary>
        /// Добавления типа двигателя и его количество в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddTypeOfEntity_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            Color colorBody = Color.FromArgb(rnd.Next() % 256, rnd.Next() % 256, rnd.Next() % 256);
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                colorBody = dialog.Color;
            }
            var entity = new EntityAirplane((int)numericSpeed.Value, (int)numericWeight.Value, colorBody);
            _generatorAirplane.AddTypeOfEntity(entity);
            MessageBox.Show($"Добавлены свойства самолета:\n" +
                $"Вес: {entity.Weight}\n" +
                $"Скорость: {entity.Speed}\n" +
                $"Цвет: {colorBody.Name}", 
                "Успешно добавлены свойства");
        }
        /// <summary>
        /// Генерация самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGenerateAirplane_Click(object sender, EventArgs e)
        {
            AddAirplaneInMap(_generatorAirplane.Generate(), 
                "Не удалось сгенерировать самолет. Добавьте хотя бы по одному количество двигателей и свойств для генерации", 
                "Генерация самолета"
                );
        }
        /// <summary>
        /// Добавления сущности в генератор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAddTypeOfEngines_Click(object sender, EventArgs e)
        {
            IAirplaneEngines? typeAirplaneEngines = null;
            switch (comboTypeEngines.Text)
            {
                case "Квадратный":
                    typeAirplaneEngines = new AirplaneRectEngines();
                    break;
                case "Стрелка":
                    typeAirplaneEngines = new AirplaneArrowEngines();
                    break;
                default:
                    typeAirplaneEngines = new DrawningAirplaneEngines();
                    break;
            }
            typeAirplaneEngines.CountEngines = (int)numericUpDownEngines.Value;
            _generatorAirplane.AddTypeOfEngines(typeAirplaneEngines);
        }
        /// <summary>
        /// Вызов формы для создания самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddAirplane_Click(object sender, EventArgs e)
        {
            FormAirplaneConfig formAirplane = new();
            formAirplane.AddEvent(new(AddAirplane));
            formAirplane.Show();
        }
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="airplane"></param>
        private void AddAirplane(DrawningAirplane airplane)
        {
            AddAirplaneInMap(new(airplane));
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveAirplane_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1 || string.IsNullOrEmpty(maskedTextBoxPosition.Text)
                || MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                var deletedObject = _mapsCollection[NameMap] - pos;
                if (deletedObject != null)
                {
                    MessageBox.Show("Объект удален");
                    _logger.LogInformation("Из текущей карты удален объект {@Airplane}", deletedObject);
                    _deletedObjects.AddLast(deletedObject);
                    pictureBox.Image = _mapsCollection[NameMap].ShowSet();
                }
                else
                {
                    _logger.LogInformation("Не удалось добавить объект по позиции {0} равен null", pos);
                    MessageBox.Show("Не удалось удалить объект");
                }
            }
            catch (AirplaneNotFoundException ex)
            {
                _logger.LogWarning("Ошибка удаления: {0}", ex.Message);
                MessageBox.Show($"Ошибка удаления: {ex.Message}");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[NameMap].ShowSet();
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[NameMap].ShowOnMap();
        }

        private void btnAddDeletedObject_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (_deletedObjects.Count == 0)
            {
                MessageBox.Show("Нет удаленных объектов", "Добавление удаленного объекта");
                return;
            }
            FormAirBomber form = new((DrawningAirplane)_deletedObjects.First());
            _deletedObjects.RemoveFirst();
            if (form.ShowDialog() == DialogResult.OK)
            {
                AddAirplaneInMap(form.SelectedAirplane);
            }
        }
        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    _logger.LogInformation("Сохранение прошло успешно. Файл находится: {0}", saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Не удалось сохранить файл '{0}'. Текст ошибки: {1}", saveFileDialog.FileName, ex.Message);
                }

            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Открытие прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadMaps();
                }
                catch (Exception ex)
                {
                    _logger.LogError("Не удалось открыть файл {0}. Текст ошибки: {1}", openFileDialog.FileName, ex.Message);
                    MessageBox.Show("Не удалось открыть", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void SaveMapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog.FileName = NameMap;
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Открытие прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Открытие файла '{0}' прошло успешно", openFileDialog.FileName);
                    ReloadMaps();
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не удалось открыть: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Не удалось открыть файл {0}. Текст ошибки: {1}", openFileDialog.FileName, ex.Message);
                }

            }
        }
    }
}